unit USocket;
{
Beispielprogram des Tutorials "Sockets mit WinAPI"
(c) Etienne Gasch 2007
}

interface

uses
  Windows, Messages, SysUtils, Forms, Dialogs,
  XPMan, StdCtrls, Classes, WinSock, Controls, Sockets;

const MainCaption='IP - Sockets'; //caption of formular
const BooltoStr:array[false..true] of string=('disabled','enabled');

type TPart =(pServer,pClient,pNone);
     TTransport=(tTCP,TUDP,tNone);

type
  TSocketForm = class(TForm)
    CreateSocketButton: TButton;
    BindSocketButton: TButton;
    ListenButton: TButton;
    ConnectButton: TButton;
    SendButton: TButton;
    DisconnectButton: TButton;
    SendEdit: TEdit;
    Memo: TMemo;
    CloseProgramButton: TButton;
    TypeMemo: TMemo;
    ValueMemo: TMemo;
    XPManifest1: TXPManifest;
    EventButton: TButton;
    BroadcastCheckBox: TCheckBox;
    AliveCheckBox: TCheckBox;
    BufferButton: TButton;
    ScrollBar: TScrollBar;

    procedure FormCreate(Sender: TObject);

    procedure CreateSocketButtonClick(Sender: TObject);

    procedure BindSocketButtonClick(Sender: TObject);
    procedure ListenButtonClick(Sender: TObject);
    procedure ConnectButtonClick(Sender: TObject);
    procedure EventButtonClick(Sender: TObject);

    procedure SendButtonClick(Sender: TObject);

    procedure DisconnectButtonClick(Sender: TObject);

    procedure CloseProgramButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);


    procedure BroadcastCheckBoxClick(Sender: TObject);
    procedure AliveCheckBoxClick(Sender: TObject);
    procedure BufferButtonClick(Sender: TObject);
    procedure ScrollBarChange(Sender: TObject);

  private
    FSocket:TSocket;
    FServerConnect:TSocket;

    FPart:TPart;
    FTransport:TTransport;
    FIP:string;
    FPort:integer;

    FwinSockstarted:boolean;

    procedure SocketMessage(var msg:Tmessage);message WM_APP;

    procedure Error(ErrorCode:integer;action:string='');

    function SocketAddressInput(out SockAddr:TSockAddrIn;forBinding:boolean):boolean;
    function TransPortLayerInput:TTransport;

    function sendTCP:integer;
    function sendUDP:integer;
    procedure transferBuffersizes;

    procedure FillList(pos:integer=-1);
    procedure DisConnectClient;
    procedure DisConnectSocket;
  end;

var
  SocketForm: TSocketForm;

implementation

uses UAddressForm, // enter IP and port
     ULayerForm, // choose between TCP and UDP
     UBufferForm; //input new buffer values for socket


{$R *.dfm}

//***************** functions for special services *********

procedure TSocketForm.Error(ErrorCode:integer;action:string='');
//common error function
//ErrorCode is mostly from WSAgetlastError
//action belongs to the message string
var s,top:string;
begin
  top:=self.Caption;
  delete(top,1,length(mainCaption));
  while (length(top)>0)and(top[1]=' ') do delete(top,1,1);
  if length(top)>0 then
    top:='socket on: '+top+#13#10#13#10
  else
    top:='';
  s:=syserrormessage(errorCode);
  if length(action)=0 then
    showmessage(top+'Error occured - Error number: $'+
                inttohex(errorcode,8)+#13#10+s)
  else
    showmessage(top+'Error occured while '+action+
                ' - Error number: $'+inttohex(errorcode,8)+#13#10+s);
end;


procedure TSocketForm.FillList(pos:integer=-1);
//creates the info box on the upper right corner
//if Socket is TCP-Server with connected Client..
//..pos describes the page to be shown (Server- or Client-Socket)

//the infobox consists of two TMemo without any border

//the infos are taken via
//getsockname
//getpeername
//gethostbyname
//and different getsockopt calls
var SocketAddr:TSockAddr;
    len,I_optvalue:integer;
    B_optvalue:boolean;
    aSocket:TSocket;
    Hostent:PHostent;
    addr:^PInAddr;
    Errorcode:integer;
    nCaption:string;
begin
  nCaption:=MainCaption+'   ';
  typememo.Clear;
  valuememo.Clear;
  if FServerConnect=INVALID_SOCKET then
  begin
    aSocket:=FSocket;
    scrollbar.Visible:=false;
  end
  else
  begin
    scrollbar.Visible:=true;
    if pos<0 then pos:=scrollbar.Position;
    if pos=0 then
    begin
      aSocket:=FServerConnect;
      valuememo.Lines.add('Connect-Socket');
    end
    else
    begin
      aSocket:=FSocket;
      valuememo.Lines.add('Listen-Socket')
    end;
    valuememo.Lines.add('-----------------------');
    typememo.lines.add('');
    typememo.lines.add('');
  end;


  len:=sizeof(SocketAddr);
  ErrorCode:=getsockname(aSocket,socketAddr,len);
  if ErrorCode<>Socket_Error then
  begin
    typememo.lines.Add('Socket Address:');
    valuememo.Lines.Add(inet_ntoa(socketaddr.sin_addr));
  end;
  HostEnt:=gethostbyname(nil);
  if HostEnt<>nil then
  begin
    Addr:=Pointer(hostEnt^.h_addr_list);
    if (Addr<>nil) AND (Addr^<>nil) then
    begin
      typememo.lines.Add('Local IP Address:');
      valuememo.Lines.Add(inet_ntoa(Addr^^));
      ncaption:=ncaption+inet_ntoa(Addr^^);
    end;
    typememo.Lines.Add('Local Host:');
    valuememo.Lines.add(hostEnt^.h_name);
  end;
  if ErrorCode<>Socket_Error then
  begin
    typememo.Lines.Add('Local Port:');
    valuememo.Lines.add(inttostr(ntohs(socketaddr.sin_port)));
    ncaption:=ncaption+':'+inttostr(ntohs(socketaddr.sin_port));
  end;
  len:=sizeof(SocketAddr);
  Errorcode:=getpeername(aSocket,SocketAddr,len);
  if ErrorCode<>Socket_Error then
  begin
    typememo.lines.Add('Remote IP Address:');
    valuememo.Lines.Add(inet_ntoa(socketaddr.sin_addr));
    typememo.Lines.Add('Remote Port:');
    valuememo.Lines.add(inttostr(ntohs(socketaddr.sin_port)));
  end;

  typememo.Lines.add('');
  valuememo.Lines.Add('');

  len:=4;
  if getsockopt(aSocket,SOL_SOCKET,SO_SNDBUF,@I_optvalue,len)<>Socket_Error then
  begin
    typememo.Lines.Add('sendbuffer in bytes:');
    valuememo.Lines.Add(inttostr(I_optvalue));
  end;
  len:=4;
  if getsockopt(aSocket,SOL_SOCKET,SO_RCVBUF,@I_optvalue,len)<>Socket_Error then
  begin
    typememo.Lines.Add('recievebuffer in bytes:');
    valuememo.Lines.Add(inttostr(I_optvalue));
  end;
  len:=1;
  if getsockopt(aSocket,SOL_SOCKET,SO_Broadcast,@B_optvalue,len)<>Socket_Error then
  begin
    typememo.Lines.Add('broadcasting:');
    valuememo.Lines.Add(booltostr[B_optvalue]);
  end;
  len:=1;
  if getsockopt(aSocket,SOL_SOCKET,SO_Acceptconn,@B_optvalue,len)<>Socket_Error then
  begin
    typememo.Lines.Add('listening:');
    valuememo.Lines.Add(booltostr[B_optvalue]);
  end;
  len:=1;
  if getsockopt(aSocket,SOL_SOCKET,SO_KeepAlive,@B_optvalue,len)<>Socket_Error then
  begin
    typememo.Lines.Add('sending "keep alives":');
    valuememo.Lines.Add(booltostr[B_optvalue]);
  end;
  len:=1;
  if getsockopt(aSocket,SOL_SOCKET,SO_ReuseAddr,@B_optvalue,len)<>Socket_Error then
  begin
    typememo.Lines.Add('reuse socket address:');
    valuememo.Lines.Add(booltostr[B_optvalue]);
  end;


  //change also the caption of the form
  case FTransport of
    tTCP:begin
      ncaption:=ncaption+'  TCP';
      case Fpart of
        pServer:ncaption:=ncaption+'(server)';
        pClient:ncaption:=ncaption+'(client)';
      end;
    end;
    tUDP:ncaption:=ncaption+'  UDP';
  end;
  self.caption:=ncaption;
end;


function TSocketForm.SocketAddressInput(out SockAddr:TSockAddrIn; forBinding:boolean):boolean;
//calls TAddressform from U_Addressform
//user enters the IP and port address
//gives back the address via SockAddr
//forbinding: True if the importform is used for binding
//while binding IP 255.255.255.255 is not allowed
//result represents a valid input
var Port:integer;
    AddressForm:TAddressForm;
begin
  AddressForm:=TAddressForm.Create(self);
  try
    Addressform.IPMaskEdit.Text:=FIP;
    Addressform.PortMaskEdit.Text:=inttostr(FPort);
    result:=Addressform.ShowModal=mrok;
    if result then
    begin
      SockAddr.sin_family:=af_inet;
      result:=trystrtoint(Addressform.PortMaskEdit.Text,Port);
      result:=result and ((Port and $0000FFFF) = Port);
      if not result then
        showmessage('invalid port number')
      else
      begin
        FIP:=Addressform.IPMaskEdit.Text;
        FPort:=Port;
        SockAddr.sin_port:=htons(Port);
        SockAddr.sin_addr.S_addr:=inet_addr(Pchar(FIP));
        result:=(SockAddr.sin_addr.S_addr<>INADDR_NONE)or
               ((Addressform.IPMaskEdit.Text='255.255.255.255')and
                 (not forBinding)) ;
        if not result then
          showmessage('invalid IP address');
      end;
    end;
  finally
    AddressForm.Free;
  end;
end;

procedure Tsocketform.transferbuffersizes;
//only for TCP-Server
//transfers the enterd buffer sizes from a Listen-Socket
// to a connect-Socket
var recvbuf,sendbuf,len:integer;
begin
  len:=4;
  if getsockopt(FSocket,SOL_SOCKET,SO_SNDBUF,@sendbuf,len)=Socket_Error then
  begin
    len:=4;
    setsockopt(FServerConnect,SOL_SOCKET,SO_SNDBUF,@sendbuf,len);
  end;

  len:=4;
  if getsockopt(FSocket,SOL_SOCKET,SO_RCVBUF,@recvbuf,len)=Socket_Error then
  begin
    len:=4;
    setsockopt(FserverConnect,SOL_SOCKET,SO_RCVBUF,@recvbuf,len);
  end;
end;

function TSocketForm.TransPortLayerInput:TTransport;
//shows the LayerForm from Unit U_Layerform
//user can choose between TCP and UDP
var LayerForm:TLayerForm;
begin
  result:=tNone;
  LayerForm:=TLayerform.create(self);
  try
    if Layerform.showmodal = mrOk then
    begin
      if Layerform.TCPRadioButton.Checked then
        result:=tTCP;
      if Layerform.UDPRadioButton.Checked then
        result:=tUDP;
    end;
  finally
    Layerform.free;
  end;
end;

//*************** onCreate ****************************

procedure TSocketForm.FormCreate(Sender: TObject);
//init WinSock DLL and raises info on Versions from DLL
//init a few variables
var WSData:WSAData;
    res:integer;
begin
  Caption:=mainCaption;
  FServerConnect:=InValid_socket;
  FSocket:=InValid_socket;
  res:=wsaStartup(makeword(1,1),WSData);
  FWinSockStarted:=true;
  if res<>0 then
  begin
    error(res,'initialising');
    Fwinsockstarted:=false;
    postmessage(handle,wm_close,0,0);
  end else
    showmessage('Version '+inttostr(lobyte(WSData.wVersion))+'.'+
                inttostr(hibyte(WSData.wVersion))+
                ' der Windows Sockets DLL gestartet.'+#13#10+
                'Version '+inttostr(lobyte(WSData.wHighVersion))+'.'+
                inttostr(hibyte(WSData.wHighVersion))+
                ' wre mglich gewesen.');

  Fpart:=pNone;
  Ftransport:=tNone;
  filllist;
end;

//**************function of Button Events *******************

procedure TSocketForm.CreateSocketButtonClick(Sender: TObject);
// WinAPI-function: socket
begin
  FTransport:=TransportLayerInput;
  if FTransport<>tNone then
  begin
    case FTransport of
      tTCP:FSocket:=Socket(AF_Inet,SOCK_STREAM,IPProto_IP);
      tUDP:FSocket:=Socket(AF_Inet,SOCK_DGRAM,IPProto_IP);
      else FSocket:=invalid_Socket;
    end;

    if FSocket<>invalid_socket then
    begin
      CreateSocketButton.Enabled:=false;
      BindSocketButton.Enabled:=true;
      Bufferbutton.Enabled:=true;
      FIP:='0.0.0.0';
      FPort:=21000;
      case FTransport of
        tTCP:ConnectButton.Enabled:=true;
        tUDP:begin
          sendbutton.Enabled:=true;
          broadcastCheckbox.Enabled:=true;
        end;
      end;
      DisConnectButton.enabled:=true;
      showmessage('socket created successfully');
    end
    else
    begin
      error(WSAGetlasterror,'creating socket');
    end;
  end;
  FillList;
end;


procedure TSocketForm.BindSocketButtonClick(Sender: TObject);
//WinAPI-function: bind
var SocketAddr:TSockAddrIn;
begin
  if SocketAddressInput(SocketAddr,true) then
  begin
    if bind(FSocket,SocketAddr,sizeof(SocketAddr))=Socket_ERROR then
    begin
      error(WSAGetLastError,'binding socket');
    end
    else
    begin
      BindSocketButton.Enabled:=false;
      case FTransport of
        tTCP: ListenButton.enabled:=true;
        tUDP: eventButton.enabled:=true;
      end;
      FIP:='127.0.0.1';

      showmessage('socket bound to: '+
                   inet_ntoa(SocketAddr.sin_addr)+
                   ':'+inttostr(ntohs(SocketAddr.sin_port)));
    end;
  end;
  FillList;
end;


procedure TSocketForm.ListenButtonClick(Sender: TObject);
//WinAPI-function: listen
begin
  if listen(FSocket,SOMAXCONN) = SOCKET_ERROR then
  begin
    error(WSAgetLastError,'switching to listen');
  end
  else
  begin
    listenbutton.enabled:=false;
    connectbutton.enabled:=false;
    eventbutton.enabled:=true;
    showmessage('listening on Socket');
    Fpart:=pServer;
  end;
  fillList;
end;


procedure TSocketForm.ConnectButtonClick(Sender: TObject);
//WinAPI-function: connect
var SocketAddr:TSockAddr;
begin
  if FIP='0.0.0.0' then FIP:='127.0.0.1';
  if SocketAddressInput(SocketAddr,false) then
  begin
    if connect(FSocket,SocketAddr,sizeof(SocketAddr))=Socket_ERROR then
    begin
      error(WSAgetLastError,'connecting');
      bindsocketbutton.Enabled:=false;
    end
    else
    begin
      listenbutton.enabled:=false;
      connectbutton.enabled:=false;
      bindSocketButton.Enabled:=false;
      sendButton.Enabled:=true;
      eventbutton.Enabled:=true;
      alivecheckbox.Enabled:=true;
      showmessage('Connected to: '+
                   inet_ntoa(SocketAddr.sin_addr)+
                   ':'+inttostr(ntohs(SocketAddr.sin_port)));
      Fpart:=pclient;

    end;
  end;
  FillList;
end;

procedure TSocketForm.EventButtonClick(Sender: TObject);
//WinAPI-function: WSAAsyncSelect
begin
  if WSAAsyncSelect(FSocket,handle,WM_APP,FD_ACCEPT or
                                       FD_READ or
                                       FD_WRITE or
                                       FD_CONNECT or
                                       FD_CLOSE) = socket_error then
  begin
    error(WSAgetlasterror,'setting event');
  end
  else
  begin
    eventbutton.Enabled:=false;
    showmessage('event set');
  end;
end;

procedure TSocketForm.SendButtonClick(Sender: TObject);
//WinAPI-function: send or sendto
//is called via sendTCP or sendUDP
var res:integer;
begin
  case FTransport of
    tTCP:res:=sendTCP;
    tUDP:res:=sendUDP;
    else res:=0;
  end;
  if res=Socket_Error then
  begin
    error(WSAgetLastError,'sending');
  end
  else if res>=0 then
  begin
    showmessage('message sent (bytes: '+inttostr(res)+')');
    sendedit.Text:='';
  end;
  filllist;
end;

procedure TSocketForm.DisconnectButtonClick(Sender: TObject);
//WinAPI-function: closesocket
//via DisconnectClient and DisconnectSocket
begin
  DisconnectClient;
  DisConnectSocket;
  Bufferbutton.Enabled:=false;
  FTransport:=tNone;
end;

procedure TSocketForm.BufferButtonClick(Sender: TObject);
//change socket options via
//WinAPI-function: setsockopt
var BufferForm:TBufferForm;
    recvbuf,sendbuf,len:integer;
    aSocket:Tsocket;
begin
  if FServerConnect=INVALID_SOCKET then aSocket:=FSocket
  else begin
    if scrollbar.Visible then
    begin
      if scrollbar.position=0 then asocket:=FServerConnect
                              else aSocket:=FSocket;
    end
    else
      aSocket:=FServerConnect;
  end;

  len:=4;
  if getsockopt(aSocket,SOL_SOCKET,SO_SNDBUF,@sendbuf,len)=Socket_Error then
  begin
    error(wsagetlasterror,'getting socket options');
    exit;
  end;
  len:=4;
  if getsockopt(aSocket,SOL_SOCKET,SO_RCVBUF,@recvbuf,len)=Socket_Error then
  begin
    error(wsagetlasterror,'getting socket options');
    exit;
  end;

  Bufferform:=Tbufferform.Create(self);
  try
    bufferform.sendMaskEdit.Text:=inttostr(sendbuf);
    bufferform.recvMaskEdit.Text:=inttostr(recvbuf);
    if bufferform.ShowModal=mrok then
    begin
      if trystrtoint(bufferform.sendMaskEdit.Text,sendbuf)and
         trystrtoint(bufferform.recvMaskEdit.Text,recvbuf) then
      begin
        if (sendbuf>0)and(recvbuf>0) then
        begin
          len:=4;
          if setsockopt(aSocket,SOL_SOCKET,SO_SNDBUF,@sendbuf,len)=Socket_Error then
          begin
            error(wsagetlasterror,'setting socket options "send buffer"');
          end;
          len:=4;
          if setsockopt(aSocket,SOL_SOCKET,SO_RCVBUF,@recvbuf,len)=Socket_Error then
          begin
            error(wsagetlasterror,'setting socket options "recieve buffer"');
          end;
          filllist;
        end
        else
          showmessage('Error in buffer size (must be larger than 0)');
      end
      else
        showmessage('Error in buffer size (no number)');
     end;
  finally
    bufferform.Free;
  end;
end;

procedure TSocketForm.CloseProgramButtonClick(Sender: TObject);
//only close and use "closesocket"
begin
  close;
end;

//************* Checkboxes changed ********************

procedure TSocketForm.BroadcastCheckBoxClick(Sender: TObject);
//WinAPI-function: setsockopt
var value:boolean;
    len:integer;
begin
  if FSocket<>invalid_socket then
  begin
    len:=1;
    value:=broadcastCheckbox.Checked;
    if setsockopt(FSocket,SOL_SOCKET,SO_BROADCAST,@value,len)=Socket_Error then
    begin
      Error(WSAgetlasterror,'setting broadcast option');
    end
    else
    begin
      showmessage('specified broadcast option set ('
                   +booltostr[value]+')');
    end;
    filllist;
  end;
end;

procedure TSocketForm.AliveCheckBoxClick(Sender: TObject);
//WinAPI-function: setsockopt
var len:integer;
    value:boolean;
    aSocket:TSocket;
begin
  if FServerConnect=INVALID_SOCKET then aSocket:=FSocket
                                   else aSocket:=FServerConnect;
  if aSocket<>INVALID_SOCKET then
  begin
    len:=1;
    value:=aliveCheckbox.Checked;
    if setsockopt(aSocket,SOL_SOCKET,SO_keepalive,@value,len)=Socket_Error then
    begin
      Error(WSAgetlasterror,'setting "keep alive" option');
    end
    else
    begin
      showmessage('specified "keep alive" option set ('
                   +booltostr[value]+')');
    end;
    filllist;
  end;
end;


//************** other functions ************************

procedure TSocketForm.SocketMessage(var msg:Tmessage);
//function for getting Messages from Socket
//WinAPI-function: recv and recvfrom
var aSocket:TSocket;
    FD,MError:word;
    SocketAddr:TSockAddr;
    len,recvBytes:integer;
    buf:array[0..30] of char;
begin
  aSocket:=msg.WParam;
  FD:=msg.LParamlo;
  MError:=msg.LParamHi;
  case FD of
    FD_ACCEPT: begin
      len:=sizeof(TSockAddr);
      if FServerConnect=INVALID_socket then
      begin
        FServerConnect:=accept(aSocket,@SocketAddr,@len);
        if FServerConnect=INVALID_socket then
          error(WSAGetLastError,'accepting')
        else
        begin
          memo.Lines.Add('client accept from: '+
                          inet_ntoa(SocketAddr.sin_addr)+
                          ':'+inttostr(ntohs(SocketAddr.sin_port)));
          sendbutton.Enabled:=true;
          alivecheckbox.Enabled:=true;
          transferbuffersizes;
        end;
      end
      else
      begin
        closeSocket(accept(aSocket,@SocketAddr,@len));
        memo.Lines.Add('client denied from: '+
                        inet_ntoa(SocketAddr.sin_addr)+
                        ':'+inttostr(ntohs(SocketAddr.sin_port)));
      end;
      FillList;
    end;
    FD_READ: begin
       fillchar(buf,length(buf),0);
       len:=sizeof(SocketAddr);
       recvBytes:=recvfrom(aSocket,buf,length(buf),0,SocketAddr,len);
       case FTransport of
         tUDP:begin
           case recvBytes of
             Socket_Error: error(WSAgetLastError,'recieving');
             0: begin
               memo.lines.Add('recieving from: '+inet_ntoa(SocketAddr.sin_addr)+
                              ':'+inttostr(ntohs(SocketAddr.sin_port)));
               memo.lines.add(' -- empty message recieved --');
             end;
             else begin
               memo.lines.Add('recieving from: '+inet_ntoa(SocketAddr.sin_addr)+
                              ':'+inttostr(ntohs(SocketAddr.sin_port)));
               memo.Lines.Add('--> '+buf);
             end;
           end;
         end;
         tTCP:begin
           case recvBytes of
             Socket_Error: error(WSAgetLastError,'recieving');
             0: memo.lines.add(' -- empty message recieved --');
             else begin
               memo.lines.Add('recieving from client:');
               memo.Lines.Add('--> '+buf);
             end;
           end;
         end;
       end;
       filllist;
    end;
    FD_WRITE: memo.lines.add('can write on socket');
    FD_Connect: begin
      memo.lines.add('connect on socket');
      filllist;
    end;
    FD_Close: begin
      memo.lines.Add('disconnect recieved');
      Disconnectclient;
    end;
  end;

  if MError<>0 then error(MError);

end;


function TSocketForm.sendTCP:integer;
//WinAPI-function: send
var s:string;
    aSocket:Tsocket;
begin
  s:=sendedit.Text;
  if FServerConnect=INVALID_SOCKET then aSocket:=FSocket
                                   else aSocket:=FServerConnect;
  result:=send(aSocket,s[1],length(s),0);
end;
function TSocketForm.sendUDP:integer;
//WinAPI-function: sendto
var s:string;
    SocketAddr:TSockAddr;
    len:integer;
begin
  s:=sendedit.Text;
  if FIP='0.0.0.0' then FIP:='127.0.0.1';
  if socketAddressInput(SocketAddr,false) then
  begin
    len:=sizeof(SocketAddr);
    result:=sendto(FSocket,s[1],length(s),0,SocketAddr,len);

    if (result<>Socket_Error)and(BindSocketButton.Enabled) then
    begin
      bindSocketButton.Enabled:=false;
      eventButton.Enabled:=true;
    end;

  end
  else
    result:=-2;
end;


procedure TSocketForm.DisConnectClient;
//WinAPI-function: closesocket
var saveSocket:Tsocket;
begin
  if FServerConnect<>INVALID_Socket then
  begin
    if closeSocket(FServerConnect)<>0 then error(WSAGetLastError,'disconnecting client');
    sendbutton.Enabled:=false;
    FServerconnect:=INVALID_SOCKET;
    fillList;
  end
  else begin
    DisConnectSocket;
  end;

  saveSocket:=FSocket;
  Fsocket:=INVALID_SOCKET;
  AliveCheckbox.Checked:=false;
  AliveCheckbox.Enabled:=false;
  Fsocket:=savesocket;
end;
procedure TsocketForm.DisconnectSocket;
//WinAPI-function: closesocket
begin
  if Fsocket=invalid_socket then exit;
  if closesocket(FSocket)<>0 then error(WSAGetLastError,'disconnecting');
  FSocket:=Invalid_Socket;
  Disconnectbutton.Enabled:=false;
  Sendbutton.Enabled:=false;
  Connectbutton.Enabled:=false;
  ListenButton.Enabled:=false;
  BindSocketButton.Enabled:=false;
  CreateSocketButton.Enabled:=true;
  EventButton.Enabled:=false;
  BroadCastCheckbox.Checked:=false;
  Broadcastcheckbox.Enabled:=false;
  typeMemo.Clear;
  valuememo.Clear;
  showmessage('socket closed');
  Fpart:=pNone;
  FillList;
end;

procedure TSocketForm.ScrollBarChange(Sender: TObject);
//only for TCP-Server with two Socket-infos
begin
  filllist(scrollbar.position);
end;

//*********** OnClose *********************
procedure TSocketForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Disconnectbutton.Enabled then disconnectbuttonclick(self);
  if FwinSockstarted then wsaCleanup;
end;



end.
